"""
    in(node::Node, graph::DAG)

Check whether the node is part of the graph.
"""
in(node::Node, graph::DAG) = node in graph.nodes

"""
    in(edge::Edge, graph::DAG)

Check whether the edge is part of the graph.
"""
function in(edge::Edge, graph::DAG)
    n1 = edge.edge[1]
    n2 = edge.edge[2]
    if !(n1 in graph) || !(n2 in graph)
        return false
    end

    return n1 in n2.children
end

"""
    ==(n1::Node, n2::Node, g::DAG)

Check equality of two nodes in a graph.
"""
function ==(n1::Node, n2::Node, g::DAG)
    if typeof(n1) != typeof(n2)
        return false
    end
    if !(n1 in g) || !(n2 in g)
        return false
    end

    return n1.task == n2.task && children(n1) == children(n2)
end
