"""
    compute_effort(t::ComputeTaskS1)

Return the compute effort of an S1 task.
"""
compute_effort(t::ComputeTaskS1) = 11

"""
    compute_effort(t::ComputeTaskS2)

Return the compute effort of an S2 task.
"""
compute_effort(t::ComputeTaskS2) = 12

"""
    compute_effort(t::ComputeTaskU)

Return the compute effort of a U task.
"""
compute_effort(t::ComputeTaskU) = 1

"""
    compute_effort(t::ComputeTaskV)

Return the compute effort of a V task.
"""
compute_effort(t::ComputeTaskV) = 6

"""
    compute_effort(t::ComputeTaskP)

Return the compute effort of a P task.
"""
compute_effort(t::ComputeTaskP) = 0

"""
    compute_effort(t::ComputeTaskSum)

Return the compute effort of a Sum task. 

Note: This is a constant compute effort, even though sum scales with the number of its inputs. Since there is only ever a single sum node in a graph generated from the ABC-Model,
this doesn't matter.
"""
compute_effort(t::ComputeTaskSum) = 1

"""
    show(io::IO, t::DataTask)

Print the data task to io.
"""
function show(io::IO, t::DataTask)
    return print(io, "Data", t.data)
end

"""
    show(io::IO, t::ComputeTaskS1)

Print the S1 task to io.
"""
show(io::IO, t::ComputeTaskS1) = print(io, "ComputeS1")

"""
    show(io::IO, t::ComputeTaskS2)

Print the S2 task to io.
"""
show(io::IO, t::ComputeTaskS2) = print(io, "ComputeS2")

"""
    show(io::IO, t::ComputeTaskP)

Print the P task to io.
"""
show(io::IO, t::ComputeTaskP) = print(io, "ComputeP")

"""
    show(io::IO, t::ComputeTaskU)

Print the U task to io.
"""
show(io::IO, t::ComputeTaskU) = print(io, "ComputeU")

"""
    show(io::IO, t::ComputeTaskV)

Print the V task to io.
"""
show(io::IO, t::ComputeTaskV) = print(io, "ComputeV")

"""
    show(io::IO, t::ComputeTaskSum)

Print the sum task to io.
"""
show(io::IO, t::ComputeTaskSum) = print(io, "ComputeSum")

"""
    copy(t::DataTask)

Copy the data task and return it.
"""
copy(t::DataTask) = DataTask(t.data)

"""
    children(::DataTask)

Return the number of children of a data task (always 1).
"""
children(::DataTask) = 1

"""
    children(::ComputeTaskS1)

Return the number of children of a ComputeTaskS1 (always 1).
"""
children(::ComputeTaskS1) = 1

"""
    children(::ComputeTaskS2)

Return the number of children of a ComputeTaskS2 (always 2).
"""
children(::ComputeTaskS2) = 2

"""
    children(::ComputeTaskP)

Return the number of children of a ComputeTaskP (always 1).
"""
children(::ComputeTaskP) = 1

"""
    children(::ComputeTaskU)

Return the number of children of a ComputeTaskU (always 1).
"""
children(::ComputeTaskU) = 1

"""
    children(::ComputeTaskV)

Return the number of children of a ComputeTaskV (always 2).
"""
children(::ComputeTaskV) = 2


"""
    children(::ComputeTaskSum)

Return the number of children of a ComputeTaskSum.
"""
children(t::ComputeTaskSum) = t.children_number

"""
    children(t::FusedComputeTask)

Return the number of children of a FusedComputeTask.
"""
function children(t::FusedComputeTask)
    return length(union(Set(t.t1_inputs), Set(t.t2_inputs)))
end

function add_child!(t::ComputeTaskSum)
    t.children_number += 1
    return nothing
end
